package arcade.assignments;

import java.io.*;
import java.sql.*;
import java.util.*;

import arcade.database.*;
import arcade.util.parser.*;

public class AssignmentProposal extends arcade.database.DBObject {
	private int m_nID;
	private int m_nIDCourseInstance;
	private String m_sName;
	private String m_sType;
	private String m_sDescription;
	private String m_sRequirements;
	private String m_sLinks;
	private String m_sContentName;
	private int m_nMaxStudents = 1;
	private int m_nAssignedByInstructor;//1-yes, 0-no, no more values
	private int m_nIDUser;
	private static final String C_table_name = "ASSIGNMENTS_PROPOSALS";

	public AssignmentProposal() {
		super();
	}

	public static void delete(int nID) throws Exception {		
		Connection db = DBConnection.getDBConnection().getConnection();
		String sQuery = "delete from " + C_table_name + " where ID = " + nID;
		PreparedStatement ps = db.prepareStatement(sQuery);
		ps.executeUpdate();
		ps.close();
	}

	protected void fetchAfterInsert(CallableStatement cs) throws Exception {
		m_nID = cs.getInt(11);
	}

	protected void fetchAfterInsert(DBConnection dbConnection) throws Exception {
		ResultSet resultSet = dbConnection.query("select max(id) from " + getTableName());
		if (resultSet.next()) {
			m_nID = resultSet.getInt(1);
		}
	}

	protected java.sql.PreparedStatement getDeleteStatement(arcade.database.DBConnection dbConnection) {
		String sQuery = "delete from " + getTableName() + " where ID = " + m_nID;
		PreparedStatement ps = null;
		try {
			Connection connection = dbConnection.getConnection();
			ps = connection.prepareStatement(sQuery);
		} catch (Exception e) {
			ps = null;
		}
		return ps;
	}	

	protected CallableStatement getInsertFetchStatement(DBConnection dbConnection) {
		String sCall =
			"begin " +
			"insert into " + getTableName() + "(ID_COURSE_INST, NAME, TYPE, DESCRIPTION, REQUIREMENTS, LINKS, CONTENT_NAME, CONTENT, MAX_STUDENTS, ASSIGNED_BY_INSTRUCTOR, ID_USER)" +
			" values(?, ?, ?, ?, ?, ?, ?, empty_blob(), ?, ?, ?) returning ID into ?; " +
			"end;";

		CallableStatement cs = null;
		try {
			cs = dbConnection.getConnection().prepareCall(sCall);
			cs.setInt(1, m_nIDCourseInstance);
			cs.setString(2, m_sName);
			cs.setString(3, m_sType);
			cs.setString(4, m_sDescription);
			cs.setString(5, m_sRequirements);
			cs.setString(6, m_sLinks);
			cs.setString(7, m_sContentName);
			cs.setInt(8, m_nMaxStudents);
			cs.setInt(9, m_nAssignedByInstructor);
			cs.setInt(10, m_nIDUser);
      			cs.registerOutParameter(11, java.sql.Types.NUMERIC);
		} catch (Exception e) {
			cs = null;
		}

		return cs;
	}

	protected java.sql.PreparedStatement getInsertStatement(arcade.database.DBConnection dbConnection) {
		ResourceBundle bundle = ResourceBundle.getBundle("arcade.database.database");
		String databaseType = bundle.getString("database_type");
		String sQuery = "";
		boolean flag = true;
		if (databaseType.equals("oracle")){
			sQuery = "insert into " + getTableName() + "(ID_COURSE_INST, NAME, TYPE, DESCRIPTION, REQUIREMENTS, LINKS, CONTENT_NAME,CONTENT, MAX_STUDENTS, ASSIGNED_BY_INSTRUCTOR, ID_USER) values(?, ?, ?, ?, ?, ?, ?,empty_blob(), ?, ?, ?)";
		}
		else {
			sQuery = "insert into " + getTableName() + "(ID_COURSE_INST, NAME, TYPE, DESCRIPTION, REQUIREMENTS, LINKS, CONTENT_NAME, MAX_STUDENTS, ASSIGNED_BY_INSTRUCTOR, ID_USER) values(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
			flag = false;
		}

		PreparedStatement ps = null;
		try {
			Connection connection = dbConnection.getConnection();
			ps = connection.prepareStatement(sQuery);
			ps.setInt(1, m_nIDCourseInstance);
			ps.setString(2, m_sName);
			ps.setString(3, m_sType);
			ps.setString(4, m_sDescription);
			ps.setString(5, m_sRequirements);
			ps.setString(6, m_sLinks);
			ps.setString(7, m_sContentName);
			if (flag) {
				ps.setInt(9, m_nMaxStudents);
				ps.setInt(10, m_nAssignedByInstructor);
				ps.setInt(11, m_nIDUser);
			} else {
				ps.setInt(8, m_nMaxStudents);
				ps.setInt(9, m_nAssignedByInstructor);
				ps.setInt(10, m_nIDUser);
			}
		} catch (Exception e) {
			ps = null;
		}

		return ps;
	}	

	protected java.sql.PreparedStatement getUpdateStatement(arcade.database.DBConnection dbConnection) {
		String sQuery = "update " + getTableName() +
			" set ID_COURSE_INST = ?, NAME = ?, TYPE = ?, DESCRIPTION = ?, REQUIREMENTS = ?, LINKS = ?, CONTENT_NAME = ?, MAX_STUDENTS = ?, ASSIGNED_BY_INSTRUCTOR = ?, ID_USER = ? where ID = " + m_nID;

		PreparedStatement ps = null;
		try {
			Connection connection = dbConnection.getConnection();
			ps = connection.prepareStatement(sQuery);

			ps.setInt(1, m_nIDCourseInstance);
			ps.setString(2, m_sName);
			ps.setString(3, m_sType);
			ps.setString(4, m_sDescription);
			ps.setString(5, m_sRequirements);
			ps.setString(6, m_sLinks);
			ps.setString(7, m_sContentName);
			ps.setInt(8, m_nMaxStudents);
			ps.setInt(9, m_nAssignedByInstructor);
			ps.setInt(10, m_nIDUser);
		} catch (Exception e) {
			ps = null;
		}
		return ps;
	}

	public void initialize(int nID) throws Exception {
		DBSet cirSet = new DBSet (AssignmentProposal.class);
		cirSet.setWhereClause("ID = " + nID);
		cirSet.getDBSet(DBConnection.getDBConnection());
		if (cirSet.getSize() != 1) {
			Exception e = new Exception ("Assignments::Assignment::initialize(id);\nassignment not found");
			throw e;
		}

		m_bInDatabase = true;

		AssignmentProposal cir = (AssignmentProposal)cirSet.getDBObject(0);

		m_nID = nID;
		setCourseInstanceID(cir.getCourseInstanceID());
		setName(cir.getName());
		setType(cir.getType());
		setDescription(cir.getDescription());
		setRequirements(cir.getRequirements());
		setLinks(cir.getLinks());
		setContentName(cir.getContentName());
		setMaxStudents(cir.getMaxStudents());
		setAssignedByInstructor(cir.getAssignedByInstructor());
		setUserID(cir.getUserID());
	}
	
	public void initialize(int nIDCourseInstance, String sName, String sType, String sDescription, String sRequirements, String sLinks, String sContentName, int nMaxStudents, int nAssignedByInstructor, int nIDUser) {
		m_bInDatabase = false;

		setCourseInstanceID(nIDCourseInstance);
		setName (sName);
		setType(sType);
		setDescription(sDescription);
		setRequirements(sRequirements);
		setLinks(sLinks);
		setContentName(sContentName);
		setMaxStudents(nMaxStudents);
		setAssignedByInstructor(nAssignedByInstructor);
		setUserID(nIDUser);
	}
	
	public void initialize(java.sql.ResultSet resultSet) throws SQLException{
		m_bInDatabase = true;

		m_nID = resultSet.getInt("ID");
		m_nIDCourseInstance = resultSet.getInt("ID_COURSE_INST");
		m_sName = resultSet.getString ("NAME");
		m_sType = resultSet.getString ("TYPE");
		m_sDescription = resultSet.getString ("DESCRIPTION");
		m_sRequirements = resultSet.getString ("REQUIREMENTS");
		m_sLinks = resultSet.getString ("LINKS");
		m_sContentName = resultSet.getString("CONTENT_NAME");
		m_nMaxStudents = resultSet.getInt("MAX_STUDENTS");
		m_nAssignedByInstructor = resultSet.getInt("ASSIGNED_BY_INSTRUCTOR");
		m_nIDUser = resultSet.getInt("ID_USER");
	}

	public void uploadFile(DBConnection dbConnection, ArcadeFile file) throws Exception {
		if (m_bInDatabase) {
			saveBlob(dbConnection, "CONTENT", "ID = " + String.valueOf(m_nID), file);		
		} else {
			throw new Exception("Cannot upload file - the file doesn't exist in the database");
		}
	}

	public long getSize() throws Exception {
		if (m_bInDatabase) {
			return getBlobSize(DBConnection.getDBConnection(), "CONTENT", "ID = " + String.valueOf(m_nID));
		} else {
			throw new Exception("Cannot get node size - the content file doesn't exist in the database");
		}
	}

	public void deleteContent(DBConnection dbConnection) throws Exception {
		ResourceBundle bundle = ResourceBundle.getBundle("arcade.database.database");
		String databaseType = bundle.getString("database_type");
		String sQuery = "";
		boolean flag = true;
		if (databaseType.equals("oracle")){
			sQuery = "update " + getTableName() +
					" set CONTENT_NAME = NULL, CONTENT = empty_blob() where ID = " + m_nID;
		}
		else {
			sQuery = "update " + getTableName() +
					" set CONTENT_NAME = NULL, CONTENT = NULL where ID = " + m_nID;
		}
	    
		PreparedStatement ps = null;
		Connection connection = dbConnection.getConnection();
		ps = connection.prepareStatement(sQuery);
		ps.executeUpdate();
		ps.close();
	}

	public void sendFileToStream(OutputStream outputStream) throws Exception {
		if (m_bInDatabase) {
			String sQuery = "select CONTENT from " + getTableName() + " where ID = " + String.valueOf(m_nID);
			ResultSet rs = DBConnection.query(sQuery);
			if (rs.next()) {
				InputStream inputStream = rs.getBinaryStream(1);
				final int nBufferSize = 1024;
				byte[] buffer = new byte[nBufferSize];
				int nBytesRead = inputStream.read(buffer, 0, nBufferSize);
				while (nBytesRead > 0) {
					outputStream.write(buffer, 0, nBytesRead);
					nBytesRead = inputStream.read(buffer, 0, nBufferSize);
				}
				inputStream.close();
			}
			DBConnection.closeResultSet(rs);
		} else {
			throw new Exception("Cannot get file - the file doesn't exist in the database");
		}
	}
	
	public String getTableName() {
		return C_table_name;
	}

	public int getID() {
		return m_nID;
	}

	public int getCourseInstanceID() {
		return m_nIDCourseInstance;
	}

	public void setCourseInstanceID(int n_IDCourseInstance) {
		m_nIDCourseInstance = n_IDCourseInstance;
	}	

	public String getName() {
		return m_sName;
	}

	public void setName(String newName) {
		m_sName = newName;
	}	

	public String getType() {
		return m_sType;
	}

	public void setType(String sType) {
		m_sType = sType;
	}	

	public String getDescription() {
		return m_sDescription;
	}

	public void setDescription(String sDescription) {
		m_sDescription = sDescription;
	}	

	public String getRequirements() {
		return m_sRequirements;
	}

	public void setRequirements(String sRequirements) {
		m_sRequirements = sRequirements;
	}	

	public String getLinks() {
		return m_sLinks;
	}

	public void setLinks(String sLinks) {
		m_sLinks = sLinks;
	}
	
	public String getContentName() {
		return m_sContentName;
	}

	void setContentName(String sContentName) {
		m_sContentName = sContentName;
	}	

	public int getMaxStudents() {
		return m_nMaxStudents;
	}

	public void setMaxStudents(int nMaxStudents) {
		m_nMaxStudents = nMaxStudents;
	}	

	public int getAssignedByInstructor() {
		return m_nAssignedByInstructor;
	}

	public void setAssignedByInstructor(int nAssignedByInstructor) {
		m_nAssignedByInstructor = nAssignedByInstructor;
	}

	public int getUserID() {
		return m_nIDUser;
	}

	public void setUserID(int nIDUser) {
		m_nIDUser = nIDUser;
	}
}