package arcade.assignments;

import java.io.*;
import java.util.*;

import arcade.util.*;
import arcade.util.html.*;
import arcade.util.parser.*;
import arcade.database.*;
import arcade.usermanagement.*;

import arcade.util.i18n.MessagesBundle;
import arcade.ArcadeConstants;

public class AssignmentProposalDlg extends Dialog {
	class AssEditData {
		long editStartTime;
		long idUserEditing;
	}

	static public final int c_TYPE_CREATE = 0;
	static public final int c_TYPE_EDIT = 1;
	static public final int c_TYPE_VIEW = 2;
	static public final int c_TYPE_DELETE = 3;
	
	static private Hashtable assEditInProgress = new Hashtable();
	static private final long TIME_OUT = 10*60*1000;
	
	int m_nType = c_TYPE_VIEW;
	int m_nAction = 0;
	transient int m_nIDCourseInstance = -1;
	transient int m_nIDAssignmentProposal = -1;
	transient int m_nIDUser = -1;
	
	public AssignmentProposalDlg(Dialog dlg, int nType) {
		super("AssignmentProposalDlg", null, "multipart/form-data", dlg);
		m_nType = nType;
	}
	
	protected void onMapVar() {
		mapVar("m_nIDCourseInstance", "CourseInstance", c_VAR_INOUT);
		mapVar("m_nIDAssignmentProposal", "AssignmentProposal", c_VAR_INOUT);
		mapVar("m_nIDUser", "User", c_VAR_INOUT);
	}
	
	//1 - iztriwane fajla sys sydyrvanieto
	//2 - izteglqne fajla sys sydyrvanieto
	//3 - zapiswane na predlovenieto w tablicata s predloveniq
	//4 - iztichane na timeout-a
	//5 - zapiswane na predlovenieto w tablicata sys zadachi, odobrenie ot instruktora
	//6 - otkaz na instruktora za predlovenieto
	
	protected void paint(WebBean wb) throws Exception {
		boolean cannotBeEdited = false;
		long currentTime = System.currentTimeMillis();	

		Integer assId = new Integer(m_nIDAssignmentProposal);
		if (assEditInProgress.containsKey(assId)) {
			AssEditData aeData = (AssEditData)assEditInProgress.get(assId);
			if (aeData.idUserEditing != wb.getLoggedUserID() && currentTime <= aeData.editStartTime + TIME_OUT) {
				cannotBeEdited = true;
			}
			if (currentTime > aeData.editStartTime + TIME_OUT) {
				assEditInProgress.remove(assId);
			}
		}

		if (!cannotBeEdited) {
			AssEditData aeData = new AssEditData();
			aeData.idUserEditing = wb.getLoggedUserID();
			aeData.editStartTime = System.currentTimeMillis();
			assEditInProgress.put(assId, aeData);
		}
		
		boolean bCreate = (m_nType == c_TYPE_CREATE);
		InputForm form = new InputForm(this);
		String sAction = wb.getRequestParameter("action");		
		boolean bError = false;
		String sErrorMessage = "";
		AssignmentProposal ap = new AssignmentProposal();
		
		if (m_nAction == 1) { //if delete operation
			try {
				if (m_nIDAssignmentProposal > 0) {					
					ap.initialize(m_nIDAssignmentProposal);					
					ap.deleteContent(DBConnection.getDBConnection());					
				}
				else 
					throw new Exception("Delete operation invoked without specifing assignment ID");
			}
			catch (Exception ex) {
				bError = true;
				sErrorMessage = MessagesBundle.getString("key_cannot_delete_content_file", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "! <BR>";
			}
			
		} else if (m_nAction == 2) { //if download operation
			try {
				if (m_nIDAssignmentProposal > 0) {
					ap.initialize(m_nIDAssignmentProposal);
					String sContentType;
					String sContentDisposition;
					int nBrowserType = WebBean.getBrowserType(wb.m_request);
					switch (nBrowserType) {
						case WebBean.C_BROWSER_TYPE_IE:
						case WebBean.C_BROWSER_TYPE_IE4:
						case WebBean.C_BROWSER_TYPE_IE5:
							sContentType = "application/x-msdownload";
							sContentDisposition = "attachment; filename=" + ap.getContentName();
							break;
						case WebBean.C_BROWSER_TYPE_IE55:
							sContentType = "application/octetstream";
							sContentDisposition = "filename=" + ap.getContentName();
							break;
						case WebBean.C_BROWSER_TYPE_NS:
							sContentType = "application/octetstream";
							sContentDisposition = "filename=" + ap.getContentName();
							break;
						case WebBean.C_BROWSER_TYPE_UNKNOWN:
						default:
							sContentType = "application/octetstream";
							sContentDisposition = "attachment; filename=" + ap.getContentName();
					}
					OutputStream outputStream = new BufferedOutputStream(wb.m_response.getOutputStream());
					Thread.yield();
					wb.m_response.setContentType(sContentType);
					wb.m_response.setContentLength((int)ap.getSize());
					wb.m_response.setHeader("Content-Disposition", sContentDisposition);
					wb.m_response.setHeader("Cache-Control", "no-cache");
					wb.m_response.setHeader("Cache-Control", "no-store");
					wb.m_response.setHeader("Pragma", "no-cache");
					ap.sendFileToStream(outputStream);
					outputStream.close();
				}
				else 
					throw new Exception("Download operation invoked without specifing assignment ID");
			
			} catch (Exception ex) {
				bError = true;
				sErrorMessage = MessagesBundle.getString("key_cannot_download_content_file", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "! <BR>";
			}
			
		} else if (m_nAction == 3) { //if write operation
			
			String sName = wb.getRequestParameter("assignmentproposalname"),
			sDescription = wb.getRequestParameter("assignmentproposaldescription"),
			sRequirements = wb.getRequestParameter("assignmentproposalrequirements"),
			sLinks = wb.getRequestParameter("assignmentproposallinks"),
			sType = wb.getRequestParameter("assignmentproposaltype");
			String sTmpMaxStudents = wb.getRequestParameter("assignmentproposalmaxstudents");
			int nMaxStudents = 1;
			try {
				nMaxStudents = Integer.parseInt(sTmpMaxStudents); 
			} catch (NumberFormatException nfe) {
				bError = true;
				sErrorMessage += MessagesBundle.getString("key_please_supply_valid_number_for_max_students", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "! <BR>";
			}
			String sTmpAssignedByInstructor = wb.getRequestParameter("assignmentproposalassignedbyinstructor");
			int nAssignedByInstructor = (sTmpAssignedByInstructor.startsWith("2 - "))?0:1; 
			sName = sName.trim();
			if (sName.length() == 0 || m_nIDCourseInstance < 1 || sTmpMaxStudents.length() == 0) {
				bError = true;
				if (sName.length() == 0) sErrorMessage += MessagesBundle.getString("key_please_supply_bank_name", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "<BR>";
				if (sTmpMaxStudents.length() == 0) sErrorMessage += MessagesBundle.getString("key_supply_max_students_for_task", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "! <BR>";
				if (m_nIDCourseInstance < 1) sErrorMessage += MessagesBundle.getString("key_bank_id_not_found", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "! <BR>"; 
			} else 
				try {
					
					ArcadeFile file = wb.getFile("assignmentproposalfile");
					if (file==null) System.out.println("dd: 1.5");
					
					if (bCreate) {
						ap.initialize(m_nIDCourseInstance, sName, sType, sDescription, sRequirements, sLinks, ((file != null) ? file.getFileName() : null), nMaxStudents, nAssignedByInstructor, (int)wb.getLoggedUserID());
						ap.insertFetch(DBConnection.getDBConnection());
						if (file != null) ap.uploadFile(DBConnection.getDBConnection(), file);
					} else {
						
						ap.initialize(m_nIDAssignmentProposal);
						ap.setCourseInstanceID(m_nIDCourseInstance);
						ap.setName(sName);					
						ap.setType(sType);
						ap.setDescription(sDescription);
						ap.setRequirements(sRequirements);
						ap.setLinks(sLinks);
						if (file != null) ap.setContentName( file.getFileName());
						ap.setMaxStudents(nMaxStudents);
						ap.setAssignedByInstructor(nAssignedByInstructor);
						ap.update(DBConnection.getDBConnection());
						if (file != null) ap.uploadFile(DBConnection.getDBConnection(), file);
					}
				} catch (Exception ex) {
					bError = true;
					sErrorMessage = MessagesBundle.getString("key_exception_while_writing_the_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + ":<BR>" + ex + "<BR>";
				}
				if (!bError) {
				OperationResult oResult = new OperationResult ();
				oResult.setTitle(bCreate ?
						  MessagesBundle.getString("key_created_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()):
						  MessagesBundle.getString("key_edited_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())
						  );
				oResult.setButtonTitle(MessagesBundle.getString("key_go_back_to_manage_assignments", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()));
				addHandler(oResult, "onCancel", c_EVENT_ON_CLICK);
				oResult.setMessage(bCreate ?
							MessagesBundle.getString("key_assignment_successfully_created", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()):
							MessagesBundle.getString("key_assignment_successfully_edited", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())
						   );
				addVisualObject(oResult);
			}			
		}
		if (m_nType == c_TYPE_DELETE) {
			m_nAction = 3;
			OperationResult oResult = new OperationResult ();
			oResult.setTitle(MessagesBundle.getString("key_deleted_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()));
			oResult.setButtonTitle(MessagesBundle.getString("key_go_back_to_manage_assignments", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()));
			addHandler(oResult, "onCancel", c_EVENT_ON_CLICK);
			oResult.setMessage(MessagesBundle.getString("key_assignment_successfully_deleted", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "!");
			try {
				if (m_nIDCourseInstance > 0) {
					ap.delete(m_nIDAssignmentProposal);										
				} else {
					oResult.setMessage(MessagesBundle.getString("key_error_while_deleting_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + m_nIDAssignmentProposal + "<BR>");
				}			
			} catch (Exception e) {
				oResult.setMessage(MessagesBundle.getString("key_error_deleting_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "!<BR>" + e + "<BR>");
			}
			addVisualObject(oResult);
		}
		if (m_nAction == 4) {
			bError = true;
			sErrorMessage = MessagesBundle.getString("key_timeout_expired", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale());
		}
		if (m_nAction == 5) {
			OperationResult oResult = new OperationResult ();
			oResult.setTitle(MessagesBundle.getString("key_approve_proposal", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()));
			oResult.setButtonTitle(MessagesBundle.getString("key_go_back_to_manage_assignments", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()));
			addHandler(oResult, "onCancel", c_EVENT_ON_CLICK);
			oResult.setMessage(MessagesBundle.getString("key_assignment_successfully_approved", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "!");
			if (wb.getRequestParameter("BankIndex") == "") {
				bError = true;
				sErrorMessage = MessagesBundle.getString("key_cannot_approve_assignment_no_bank", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "!";
			}
			if (!bError) {
				try {
					if (m_nIDCourseInstance > 0) {
						ap.delete(m_nIDAssignmentProposal);										
					} else {
						bError = true;
						oResult.setMessage(MessagesBundle.getString("key_error_while_deleting_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + m_nIDAssignmentProposal + "<BR>");
					}			
				} catch (Exception e) {
					bError = true;
					oResult.setMessage(MessagesBundle.getString("key_error_deleting_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "!<BR>" + e + "<BR>");
				}
			}
			if (!bError) {
				String sName = wb.getRequestParameter("assignmentproposalname");
				String sDescription = wb.getRequestParameter("assignmentproposaldescription");
				String sRequirements = wb.getRequestParameter("assignmentproposalrequirements");
				String sLinks = wb.getRequestParameter("assignmentproposallinks");
				String sType = wb.getRequestParameter("assignmentproposaltype");
				String sTmpMaxStudents = wb.getRequestParameter("assignmentproposalmaxstudents");
				int nMaxStudents = 1;
				try {
					nMaxStudents = Integer.parseInt(sTmpMaxStudents); 
				} catch (NumberFormatException nfe) {
					bError = true;
					sErrorMessage += MessagesBundle.getString("key_please_supply_valid_number_for_max_students", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "! <BR>";
				}
				String sTmpAssignedByInstructor = wb.getRequestParameter("assignmentproposalassignedbyinstructor");
				int nAssignedByInstructor = (sTmpAssignedByInstructor.startsWith("2 - "))?0:1; 
				int idBank = Integer.parseInt(wb.getRequestParameter("BankIndex"));
				sName = sName.trim();
				if (sName.length() == 0 || sTmpMaxStudents.length() == 0) {
					bError = true;
					if (sName.length() == 0) sErrorMessage += MessagesBundle.getString("key_please_supply_assignment_name", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "<BR>";
					if (sTmpMaxStudents.length() == 0) sErrorMessage += MessagesBundle.getString("key_supply_max_students_for_task", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "! <BR>";
				
				} else {
					try {
						ArcadeFile file = wb.getFile("assignmentproposalfile");
						Assignment a = new Assignment();
						a.initialize(idBank, sName, sType, sDescription, sRequirements, sLinks, ((file != null) ? file.getFileName() : null), nMaxStudents, nAssignedByInstructor);
						a.insertFetch(DBConnection.getDBConnection());
						if (file != null) a.uploadFile(DBConnection.getDBConnection(), file);
					} catch (Exception ex) {
						bError = true;
						sErrorMessage = MessagesBundle.getString("key_exception_while_writing_the_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + ":<BR>" + ex + "<BR>";
					}
					if (!bError) addVisualObject(oResult);
				}
			}
		}
		if (m_nAction == 6) {
			OperationResult oResult = new OperationResult ();
			oResult.setTitle(MessagesBundle.getString("key_reject_proposal", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()));
			oResult.setButtonTitle(MessagesBundle.getString("key_go_back_to_manage_assignments", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()));
			addHandler(oResult, "onCancel", c_EVENT_ON_CLICK);
			oResult.setMessage(MessagesBundle.getString("key_assignment_successfully_rejected", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "!");
			try {
				if (m_nIDCourseInstance > 0) {
					ap.delete(m_nIDAssignmentProposal);										
				} else {
					bError = true;
					oResult.setMessage(MessagesBundle.getString("key_error_while_deleting_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + m_nIDAssignmentProposal + "<BR>");
				}			
			} catch (Exception e) {
				bError = true;
				oResult.setMessage(MessagesBundle.getString("key_error_deleting_assignment", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "!<BR>" + e + "<BR>");
			}
			addVisualObject(oResult);
		}
		
		if (bError || (m_nAction == 0 || m_nAction == 1 || m_nAction == 2)) {
			
			if (bCreate)
				addVisualObject(new Header(
					MessagesBundle.getString("key_create_assignment_proposal", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					"center"));
			else addVisualObject(new Header((m_nType != c_TYPE_VIEW) ?
					MessagesBundle.getString("key_edit_assignment_proposal", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()):
					MessagesBundle.getString("key_view_assignment_proposal", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					"center"));
					
			m_nAction = 3;
			if ((!bError) && (m_nIDAssignmentProposal > 0) && (!bCreate)) {
				try {
					ap.initialize(m_nIDAssignmentProposal);					
				} catch (Exception ex) {
					bError = true;
					sErrorMessage += MessagesBundle.getString("key_exception_while_getting_the_data_for_an_assignment_proposal", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + ":<BR>" + ex + "<BR>";
				}
			}
			
			if (!cannotBeEdited) form.addInput(
					MessagesBundle.getString("key_name", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					new Input(Input.C_TYPE_TEXT, "assignmentproposalname", ap.getName()));
			else form.addInput(
					MessagesBundle.getString("key_name", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					new StaticText(ap.getName()));

			Select select = new Select("assignmentproposaltype", 1);
			select.addOption("Project", "Project", "Project".equals(ap.getType()));
			select.addOption("Essay", "Essay", "Essay".equals(ap.getType()));
			select.addOption("Problem", "Problem", "Problem".equals(ap.getType()));
			if (!cannotBeEdited) form.addInput(
					MessagesBundle.getString("key_type", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					select);
			else form.addInput("Type", new StaticText(ap.getType()));
			if (!cannotBeEdited) form.addInput(
					MessagesBundle.getString("key_description", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					new Input(Input.C_TYPE_TEXT, "assignmentproposaldescription", ap.getDescription()));
			else form.addInput(
					MessagesBundle.getString("key_description", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					new StaticText((ap.getDescription() == null)? "" : ap.getDescription()));
			TextArea textArea = new TextArea("assignmentproposalrequirements", ap.getRequirements());
			if (cannotBeEdited) textArea.setFlagForReadOnly();
			textArea.setRows(5);
			form.addInput(
					MessagesBundle.getString("key_requirements", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					textArea);
			TextArea textArea2 = new TextArea("assignmentproposallinks", ap.getLinks());
			if (cannotBeEdited) textArea2.setFlagForReadOnly();
			textArea2.setRows(5);
			form.addInput(
					MessagesBundle.getString("key_links_to_sources_of_information", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					textArea2);			
			ArrayList list = new ArrayList((!cannotBeEdited) ? 3 : 1);
			if (!cannotBeEdited) list.add(new Input(Input.C_TYPE_FILE, "assignmentproposalfile", null));
			Button bnDelete = new Button(Button.C_TYPE_BUTTON,
					MessagesBundle.getString("key_delete_content", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					"if (confirm(\"" + 
							  MessagesBundle.getString("key_delete_content_file", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) +
							  "?\")) { callback(\"onDeleteContent\"); }"),
					bnDownload = new Button(Button.C_TYPE_BUTTON, MessagesBundle.getString("key_download", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()), null);
			addHandler(bnDownload, "onDownload", c_EVENT_ON_CLICK);
			if (ap.getContentName() == null) {
				bnDelete.Disable(); bnDownload.Disable();
			}
			if (!cannotBeEdited) list.add(bnDelete);
			list.add(bnDownload);
			form.addInput((ap.getContentName() == null) ?
				MessagesBundle.getString("key_content", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()):
				MessagesBundle.getString("key_content", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + ": " + ap.getContentName(), list);
			
			if (!cannotBeEdited) {
				form.addInput(MessagesBundle.getString("key_max_students", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
						new Input(Input.C_TYPE_TEXT, "assignmentproposalmaxstudents", String.valueOf(ap.getMaxStudents())));
			} else {
				form.addInput(MessagesBundle.getString("key_max_students", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()), 
				new StaticText(String.valueOf(ap.getMaxStudents())));
			}
			
			Select select2 = new Select("assignmentproposalassignedbyinstructor", 1);
			select2.addOption(
					MessagesBundle.getString("key_assigned_by_instructor", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					MessagesBundle.getString("key_assigned_by_instructor", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					(1 == (ap.getAssignedByInstructor())));
			select2.addOption(
					MessagesBundle.getString("key_not_assigned_by_instructor", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					MessagesBundle.getString("key_not_assigned_by_instructor", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					(0 == (ap.getAssignedByInstructor())));
			if (!cannotBeEdited) {
				form.addInput(
					MessagesBundle.getString("key_way_for_assigning_the_task", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					select2);
			} else {
				form.addInput(MessagesBundle.getString("key_way_for_assigning_the_task", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
						 new StaticText( ((ap.getAssignedByInstructor()==1)?
								MessagesBundle.getString("key_assigned_by_instructor", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()):
								MessagesBundle.getString("key_not_assigned_by_instructor", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())) ));
			}
			boolean instructor = false;
			User loggedUser = wb.getLoggedUser(wb.getSession());
			if (loggedUser.isInGroup(Group.C_INSTRUCTOR_NAME)) instructor = true;
			
			Select select3 = new Select("BankIndex", 1);
			ArrayList list3 = null;
			int nCount = 0;
			if (m_nIDCourseInstance > 0) {
				list3 = ManageAssignmentsUI.getBanks(m_nIDCourseInstance, true);
				nCount = list3.size();
			}
			Bank bank;
			for (int nIndex = 0; nIndex < nCount; nIndex++) {
				bank = (Bank)list3.get(nIndex);
				select3.addOption(bank.getName(), String.valueOf(bank.getID()),false);
			}			
			if (instructor && !cannotBeEdited) {
				form.addInput(
					MessagesBundle.getString("key_bank_for_proposal", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					select3);
			}	
			ButtonBar bar = new ButtonBar();
			bar.setAlignment("center");
			Button b1 = new Button(Button.C_TYPE_BUTTON,
					MessagesBundle.getString("key_submit", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					null),
				b2 = new Button(Button.C_TYPE_BUTTON, (m_nType != c_TYPE_VIEW) ?
					MessagesBundle.getString("key_cancel", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()):
					MessagesBundle.getString("key_back", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					null),
				b3 = new Button(Button.C_TYPE_BUTTON,
					MessagesBundle.getString("key_reset_changes", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					null),
				b4 = new Button(Button.C_TYPE_BUTTON,
					MessagesBundle.getString("key_approve_proposal", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					null),
				b5 = new Button(Button.C_TYPE_BUTTON,
					MessagesBundle.getString("key_reject_proposal", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					null);
				
			
			addHandler(b1, "onSubmit", c_EVENT_ON_CLICK);
			addHandler(b2, "onCancel", c_EVENT_ON_CLICK);
			addHandler(b3, "onResetChanges", c_EVENT_ON_CLICK);
			addHandler(b4, "onApprove", c_EVENT_ON_CLICK);
			addHandler(b5, "onReject", c_EVENT_ON_CLICK);
			if (!cannotBeEdited) bar.addButton(b1);
			bar.addButton(b2);
			if (!cannotBeEdited) bar.addButton(b3);
			if (instructor && !cannotBeEdited) {
				bar.addButton(b4);
				bar.addButton(b5);
			}
			form.setButtonBar(bar);			
		}
		
		if (bError) {
			StaticText stError = new StaticText();
			stError.setText("<p align=\"center\"><font color=\"#FF0000\">" +
				sErrorMessage + "</font></p>");
			addVisualObject(0, stError);
		}
		addVisualObject(form);
	}
	
	private Dialog onDownload(WebBean wb) {
		m_nAction = 2;
		return this;
	}
	
	private Dialog onDeleteContent(WebBean wb) {
		m_nAction = 1;
		return this;
	}
	
	private Dialog onSubmit(WebBean wb) {
		Integer assId = new Integer(m_nIDAssignmentProposal);
		long currentTime = System.currentTimeMillis();
		if (assEditInProgress.containsKey(assId)) {
			AssEditData aeData = (AssEditData)assEditInProgress.get(assId);
			if (aeData.idUserEditing == wb.getLoggedUserID()) {
				assEditInProgress.remove(assId);
				if (currentTime > aeData.editStartTime + TIME_OUT) {
					m_nAction = 4;
					return this;
				}
			}
		}
		m_nAction = 3;
		return this;
	}
	
	private Dialog onCancel(WebBean wb) {
		Integer assId = new Integer(m_nIDAssignmentProposal);
		if (assEditInProgress.containsKey(assId)) {
			AssEditData aeData = (AssEditData)assEditInProgress.get(assId);
			if (aeData.idUserEditing == wb.getLoggedUserID()) {
				assEditInProgress.remove(assId);
			}
		}
		return getParent();
	}
	
	private Dialog onResetChanges(WebBean wb) {
		Integer assId = new Integer(m_nIDAssignmentProposal);
		if (assEditInProgress.containsKey(assId)) {
			AssEditData aeData = (AssEditData)assEditInProgress.get(assId);
			if (aeData.idUserEditing == wb.getLoggedUserID()) {
				assEditInProgress.remove(assId);
			}
		}
		m_nAction = 0;
		return this;
	}
	
	private Dialog onApprove(WebBean wb) {
		Integer assId = new Integer(m_nIDAssignmentProposal);
		long currentTime = System.currentTimeMillis();
		if (assEditInProgress.containsKey(assId)) {
			AssEditData aeData = (AssEditData)assEditInProgress.get(assId);
			if (aeData.idUserEditing == wb.getLoggedUserID()) {
				assEditInProgress.remove(assId);
				if (currentTime > aeData.editStartTime + TIME_OUT) {
					m_nAction = 4;
					return this;
				}
			}
		}
		m_nAction = 5;
		return this;
	}
	private Dialog onReject(WebBean wb) {
		Integer assId = new Integer(m_nIDAssignmentProposal);
		long currentTime = System.currentTimeMillis();
		if (assEditInProgress.containsKey(assId)) {
			AssEditData aeData = (AssEditData)assEditInProgress.get(assId);
			if (aeData.idUserEditing == wb.getLoggedUserID()) {
				assEditInProgress.remove(assId);
				if (currentTime > aeData.editStartTime + TIME_OUT) {
					m_nAction = 4;
					return this;
				}
			}
		}
		m_nAction = 6;
		return this;
	}
}