package arcade.assignments;

import arcade.database.DBConnection;
import arcade.database.DBSet;

import java.sql.*;

public class CalendarEvent extends arcade.database.DBObject {
	private int m_nID;
	private int m_nIDCourse = -1;
	private String m_sName;
	private String m_sType;
	private String m_sDescription;
	private java.sql.Date m_dtStartDate;
	private java.sql.Date m_dtEndDate;
	private String m_sStatus;
	private int m_nPoints;
	private int m_nMandatory;//1-yes, 0-no, no more values	
	
	private static final String C_table_name = "CALENDAR_EVENTS";
	
	public CalendarEvent() {
		super();
	}
	
	public int getID() {
		return m_nID;
	}
	
	int getIDCourse() {
		return m_nIDCourse;
	}
	public void setIDCourse(int nIDCourse) {
		m_nIDCourse = nIDCourse;
	}
	
	String getName() {
		return m_sName;
	}
	
	public void setName(String sName) {
		m_sName = sName;
	}
	
	public void setType(String sType) {
		m_sType = sType;
	}
	
	String getType() {
		return m_sType;
	}
	
	public void setDescription(String sDescription) {
		m_sDescription = sDescription;
	}
	
	String getDescription() {
		return m_sDescription;
	}
	
	java.sql.Date getStartDate() {
		return m_dtStartDate;
	}
	
	public void setStartDate(java.sql.Date dtStartDate) {
		m_dtStartDate = dtStartDate;
	}
	
	java.sql.Date getEndDate() {
		return m_dtEndDate;
	}
	
	public void setEndDate(java.sql.Date dtEndDate) {
		m_dtEndDate = dtEndDate;
	}
	
	String getStatus() {
		return m_sStatus;
	}
	
	public void setStatus(String sStatus) {
		m_sStatus = sStatus;
	}
	
	int getPoints() {
		return m_nPoints;
	}
	
	public void setPoints(int nPoints) {
		m_nPoints = nPoints;
	}
	
	int getMandatory() {
		return m_nMandatory;
	}
	
	public void setMandatory(int nMandatory) {
		m_nMandatory = nMandatory;
	}


	protected PreparedStatement getDeleteStatement(arcade.database.DBConnection dbConnection) {
		String sQuery = "delete from " + getTableName() + " where ID = " + m_nID;

		PreparedStatement ps = null;
		try {
			Connection connection = dbConnection.getConnection();
			ps = connection.prepareStatement(sQuery);
		} catch (Exception e) {
			ps = null;
		}

		return ps;
	}

	protected java.sql.PreparedStatement getUpdateStatement(arcade.database.DBConnection dbConnection) {
		String sQuery = "update " + getTableName() + 
		" set TYPE = ?, NAME = ?, DESCR = ?, COURSE_INST_ID = ?, START_DATE = ?, END_DATE = ?, STATUS = ?, POINTS = ?, CONTROLSTYLE = ? where ID = " + m_nID;

		PreparedStatement ps = null;
		try {
			Connection connection = dbConnection.getConnection();
			ps = connection.prepareStatement(sQuery);				
			ps.setString(1, m_sType);
			ps.setString(2, m_sName);
			ps.setString(3, m_sDescription);
			ps.setInt(4, m_nIDCourse);
			ps.setDate(5, m_dtStartDate);
			ps.setDate(6,  m_dtEndDate);
			ps.setString(7, m_sStatus);
			ps.setInt(8, m_nPoints);		
			ps.setInt(9, m_nMandatory);
		} catch (Exception e) {
			ps = null;
		}	
		return ps;
	}

	protected PreparedStatement getInsertStatement(arcade.database.DBConnection dbConnection) {
		String sQuery = "insert into " + getTableName() + 
			"(TYPE, NAME, DESCR, COURSE_INST_ID, START_DATE, END_DATE, STATUS, POINTS, CONTROLSTYLE) values(?, ?, ?, ?, ?, ?, ?, ?, ?)";

		PreparedStatement ps = null;
		try {
			Connection connection = dbConnection.getConnection();
			ps = connection.prepareStatement(sQuery);				
			ps.setString(1, m_sType);
			ps.setString(2, m_sName);
			ps.setString(3, m_sDescription);
			ps.setInt(4, m_nIDCourse);
			ps.setDate(5, m_dtStartDate);
			ps.setDate(6,  m_dtEndDate);
			ps.setString(7, m_sStatus);
			ps.setInt(8, m_nPoints);		
			ps.setInt(9, m_nMandatory);
		} catch (Exception e) {
			ps = null;
		}
		return ps;
	}

	protected CallableStatement getInsertFetchStatement(DBConnection dbConnection) {
		String sCall =
			"begin " +
			"insert into " + getTableName() + "(TYPE, NAME, DESCR, COURSE_INST_ID, START_DATE, END_DATE, STATUS, POINTS, CONTROLSTYLE) values(?, ?, ?, ?, ?, ?, ?, ?, ?) returning ID into ?; " +
			"end;";

		CallableStatement cs = null;
		try {
			cs = dbConnection.getConnection().prepareCall(sCall);
			cs.setString(1, m_sType);
			cs.setString(2, m_sName);
			cs.setString(3, m_sDescription);
			cs.setInt(4, m_nIDCourse);
			cs.setDate(5, m_dtStartDate);
			cs.setDate(6,  m_dtEndDate);
			cs.setString(7, m_sStatus);
			cs.setInt(8, m_nPoints);		
			cs.setInt(9, m_nMandatory);
			cs.registerOutParameter(10, java.sql.Types.NUMERIC);
		} catch (Exception e) {
			cs = null;
		}

		return cs;
	}

	protected void fetchAfterInsert(CallableStatement cs) throws Exception {
		m_nID = cs.getInt(10);
	}

	public void initialize(int nID) throws Exception {
		DBSet cirSet = new DBSet (CalendarEvent.class);
		cirSet.setWhereClause("ID = " + nID);
		cirSet.getDBSet(DBConnection.getDBConnection());
		if (cirSet.getSize() != 1) {
			Exception e = new Exception ("Assignments::Event::initialize(id);\nevent not found");
			throw e;
		}

		m_bInDatabase = true;

		CalendarEvent cir = (CalendarEvent)cirSet.getDBObject(0);
		
		m_nID = nID;
		
		setIDCourse(cir.getIDCourse());
		setName(cir.getName());
		setType(cir.getType());
		setDescription(cir.getDescription());
		setStartDate(cir.getStartDate());
		setEndDate(cir.getEndDate());
		setStatus(cir.getStatus());
		setPoints(cir.getPoints());	
		setMandatory(cir.getMandatory());	
	}

	public void initialize(String sType, String sName, String sDescription, int nIDCourse, java.sql.Date dtStartDate, java.sql.Date dtEndDate, String sStatus, int nPoints, int nMandatory) {
		m_bInDatabase = false;

		setIDCourse(nIDCourse);
		setName(sName);
		setType(sType);
		setDescription(sDescription);
		setStartDate(dtStartDate);
		setEndDate(dtEndDate);
		setStatus(sStatus);
		setPoints(nPoints);
		setMandatory(nMandatory);
	}

	public void initialize(java.sql.ResultSet resultSet) throws SQLException{
		m_bInDatabase = true;
		m_nID = resultSet.getInt("ID");
		m_sType = resultSet.getString("TYPE");
		m_sName = resultSet.getString("NAME");
		m_sDescription = resultSet.getString ("DESCR");
		m_nIDCourse = resultSet.getInt ("COURSE_INST_ID");
		m_dtStartDate = resultSet.getDate ("START_DATE");	
		m_dtEndDate = resultSet.getDate ("END_DATE");
		m_sStatus = resultSet.getString ("STATUS");
		m_nPoints = resultSet.getInt ("POINTS");
		m_nMandatory = resultSet.getInt ("CONTROLSTYLE");
	}

	public static void delete(int nID) throws Exception {
		String q = "select id from task_calendar_events where calendar_events_id = " + nID;
		ResultSet rs = DBConnection.query(q);
		if (rs.next()) {
			int id = rs.getInt(1);
			Connection db1 = DBConnection.getDBConnection().getConnection();
			String sQuery1 = "delete from task_calendar_events where ID = " + id;
			PreparedStatement ps1 = db1.prepareStatement(sQuery1);
			ps1.executeUpdate();
			ps1.close();
		}
		Connection db = DBConnection.getDBConnection().getConnection();
		String sQuery = "delete from " + C_table_name + " where ID = " + nID;
		PreparedStatement ps = db.prepareStatement(sQuery);
		ps.executeUpdate();
		ps.close();
		DBConnection.closeResultSet(rs);
		rs.close();
	}
	
	public String getTableName() {
		return C_table_name;
	}
}