package arcade.assignments;

import arcade.ArcadeConstants;
import arcade.database.DBConnection;
import arcade.util.Dialog;
import arcade.util.WebBean;
import arcade.util.html.Button;
import arcade.util.html.ButtonBar;
import arcade.util.html.Header;
import arcade.util.html.Input;
import arcade.util.html.InputForm;
import arcade.util.html.OperationResult;
import arcade.util.html.Select;
import arcade.util.html.StaticText;
import arcade.util.i18n.MessagesBundle;

public class CalendarEventDlg extends Dialog {
	
	static public final int c_TYPE_CREATE = 0;
	static public final int c_TYPE_EDIT = 1;
	static public final int c_TYPE_VIEW = 2;
	static public final int c_TYPE_DELETE = 3;
	
	int m_nType = c_TYPE_VIEW;
	int m_nAction = 0;
	transient int m_nIDCourseInstace = -1;
	transient int m_nIDCalendarEvent = -1;
		
	public CalendarEventDlg(Dialog dlg, int nType) {
		super("CalendarEventDlg", dlg);
		m_nType = nType;
	}
	
	protected void onMapVar() {
		mapVar("m_nIDCourseInstace", "CourseIndex", c_VAR_INOUT);
		mapVar("m_nIDCalendarEvent", "CalendarEvent", c_VAR_INOUT);
	}
	
	//"write" 3
	
	protected void paint(WebBean wb) throws Exception {
	
		if (!isValid("m_nIDCalendarEvent")) m_nIDCalendarEvent = -1;
		boolean bCreate = (m_nType == c_TYPE_CREATE);
		
		InputForm form = new InputForm(this);
		String sAction = wb.getRequestParameter("action");
	
		boolean bError = false;
		String sErrorMessage = "";
		
		CalendarEvent ce = new CalendarEvent();
		
		if (m_nAction == 3) {//if write operation
			String sType = wb.getRequestParameter("calendareventtype");
			String sName = wb.getRequestParameter("calendareventname");
			String sDescription = wb.getRequestParameter("calendareventdescription");
			String sStartDate = wb.getRequestParameter("calendareventstartdate");
			String sEndDate = wb.getRequestParameter("calendareventenddate");
			java.sql.Date dtStartDate = null, dtEndDate = null;
			if (sStartDate.length() > 0) {
				try {
					dtStartDate = java.sql.Date.valueOf(sStartDate);
				} catch (Exception e) {
					dtStartDate = null;
					bError = true;
					sErrorMessage += MessagesBundle.getString("key_exception_while_reading_start_date2", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "<BR>" + e + "<BR>";
				}
			}
			if (sEndDate.length() > 0) {
				try {
					dtEndDate = java.sql.Date.valueOf(sEndDate);
				} catch (Exception e) {
					dtEndDate = null;
					bError = true;
					sErrorMessage += MessagesBundle.getString("key_exception_while_reading_end_date2", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "<BR>" + e + "<BR>";
				}
			}
			if (sStartDate.equals("")) {
				bError = true; 
				sErrorMessage += MessagesBundle.getString("key_start_date_cannot_be_blank", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())
																  +"!<BR>";
			}
			if (sEndDate.equals("")) {
				bError = true; 
				sErrorMessage += MessagesBundle.getString("key_end_date_cannot_be_left_blank", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())
																  +"!<BR>";
			}
			if (dtStartDate != null && dtEndDate != null && dtStartDate.after(dtEndDate)) {
				bError = true; 
				sErrorMessage += MessagesBundle.getString("key_start_date_after_end_date", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "!<BR>";
			}
			
			String sStatus = wb.getRequestParameter("calendareventstatus");
			String sTmpPoints = wb.getRequestParameter("calendareventpoints");
			int nPoints = 0;
			try {
				nPoints = Integer.parseInt(sTmpPoints); 
			} catch (NumberFormatException nfe) {
				bError = true;
				sErrorMessage += MessagesBundle.getString("key_please_supply_valid_number_for_points", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "! <BR>";
			}
			String sTmpMandatory = wb.getRequestParameter("calendareventmandatory");
			int nMandatory = (sTmpMandatory.startsWith("1"))?1:0;			
			
			if (m_nIDCourseInstace < 1) {
				bError = true;
				sErrorMessage += MessagesBundle.getString("key_course_instance_id_not_found", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "<BR>";
			} 
			try {
				if(!bError) {
					if (bCreate) {
						ce.initialize(sType, sName, sDescription, m_nIDCourseInstace, dtStartDate, dtEndDate, sStatus, nPoints, nMandatory);
						ce.insertFetch(DBConnection.getDBConnection());
					} else {
						ce.initialize(m_nIDCalendarEvent);
						ce.setType(sType);
						ce.setName(sName);
						ce.setDescription(sDescription);
						ce.setIDCourse(m_nIDCourseInstace);
						ce.setStartDate(dtStartDate);
						ce.setEndDate(dtEndDate);
						ce.setStatus(sStatus);
						ce.setPoints(nPoints);
						ce.setMandatory(nMandatory);
						ce.update(DBConnection.getDBConnection());
					}
				}
			} catch (Exception ex) {
				bError = true;
				sErrorMessage = MessagesBundle.getString("key_exception_while_writing_the_calendar_event", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + ":<BR>" + ex + "<BR>";
			}
			
			if (!bError) {
				OperationResult oResult = new OperationResult ();
				oResult.setTitle(bCreate ?
						  MessagesBundle.getString("key_created_calendar_event", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()):
						  MessagesBundle.getString("key_edited_calendar_event", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())
						  );
				oResult.setButtonTitle(MessagesBundle.getString("key_go_back_to_manage_calendar_events", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()));
				addHandler(oResult, "onCancel", c_EVENT_ON_CLICK);
				oResult.setMessage(bCreate ?
						MessagesBundle.getString("key_calendar_event_successfully_created", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()):
						MessagesBundle.getString("key_calendar_event_successfully_edited", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())
						);
				addVisualObject(oResult);
			}
		}
		
		if (m_nType == c_TYPE_DELETE) {			
			m_nAction = 3;
			OperationResult oResult = new OperationResult ();
			oResult.setTitle(MessagesBundle.getString("key_deleted_calendar_event", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()));
			oResult.setButtonTitle(MessagesBundle.getString("key_go_back_to_manage_calendar_events", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()));
			addHandler(oResult, "onCancel", c_EVENT_ON_CLICK);
			oResult.setMessage(MessagesBundle.getString("key_calendar_event_successfully_deleted", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "!");
			try {
				ce.delete(m_nIDCalendarEvent);				
			} catch (Exception e) {
				oResult.setMessage(MessagesBundle.getString("key_error_deleting_calendar_event", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + "!<BR>" + e + "<BR>");
			}
			addVisualObject(oResult);
		}
		
		if (bError || (m_nAction != 3)) {
			if (bCreate)
				addVisualObject(new Header(
					MessagesBundle.getString("key_create_calendar_event", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					"center"));
			else addVisualObject(new Header(
					MessagesBundle.getString("key_edit_assignemnt", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),					
					"center"));
			
			m_nAction = 3;
			
			if ((!bError) && (m_nIDCalendarEvent > 0) && (!bCreate)) {
				try {
					ce.initialize(m_nIDCalendarEvent);
				} catch (Exception ex) {
					bError = true;
					sErrorMessage += MessagesBundle.getString("key_exception_while_getting_the_data_for_an_event", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + ":<BR>" + ex + "<BR>";
				}
			}
			
			Select select1 = new Select("calendareventtype", 1);
			select1.addOption("Assignment", "Assignment", "Assignment".equals(ce.getType()));
			select1.addOption("Test", "Test", "Test".equals(ce.getType()));
			form.addInput(
				MessagesBundle.getString("key_type", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()) + " *",
				select1);
				
			form.addInput(
				MessagesBundle.getString("key_name", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
				new Input(Input.C_TYPE_TEXT, "calendareventname", ce.getName()));
				
			form.addInput(
				MessagesBundle.getString("key_description", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
				new Input(Input.C_TYPE_TEXT, "calendareventdescription", ce.getDescription()));
				
			form.addInput(
				MessagesBundle.getString("key_start_date", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())+ " "+
				MessagesBundle.getString("key_yyyy_mm_dd", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())+" *",
				new Input(Input.C_TYPE_TEXT, "calendareventstartdate",  (ce.getStartDate() == null) ? null : ce.getStartDate().toString()));
				  
			form.addInput(
				MessagesBundle.getString("key_end_date", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())+ " "+
				MessagesBundle.getString("key_yyyy_mm_dd", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())+" *",
				new Input(Input.C_TYPE_TEXT, "calendareventenddate",  (ce.getEndDate() == null) ? null: ce.getEndDate().toString()));
				  
			form.addInput(
				MessagesBundle.getString("key_status", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
				new Input(Input.C_TYPE_TEXT, "calendareventstatus", ce.getStatus()));
				
			form.addInput(
				MessagesBundle.getString("key_points", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
				new Input(Input.C_TYPE_TEXT, "calendareventpoints", Integer.toString((ce.getPoints()))));
				
			Select select2 = new Select("calendareventmandatory", 1);
			select2.addOption(
				MessagesBundle.getString("key_yes_1", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
				MessagesBundle.getString("key_yes_1", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
				(1 == (ce.getMandatory())));
			select2.addOption(
				MessagesBundle.getString("key_no_1", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
				MessagesBundle.getString("key_no_1", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
				(0 == (ce.getMandatory())));
			form.addInput(
				MessagesBundle.getString("key_mandatory", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
				select2);
			
			ButtonBar bar = new ButtonBar();
			bar.setAlignment("center");
			
			Button b1 = new Button(Button.C_TYPE_BUTTON,
					MessagesBundle.getString("key_submit", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					null),
				b2 = new Button(Button.C_TYPE_BUTTON,
					MessagesBundle.getString("key_cancel", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					null),
				b3 = new Button(Button.C_TYPE_BUTTON,
					MessagesBundle.getString("key_reset_changes", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
					null);
			addHandler(b1, "onSubmit", c_EVENT_ON_CLICK);
			addHandler(b2, "onCancel", c_EVENT_ON_CLICK);
			addHandler(b3, "onResetChanges", c_EVENT_ON_CLICK);
			bar.addButton(b1);bar.addButton(b2);bar.addButton(b3);
			form.setButtonBar(bar);
		}
		if (bError) {
			StaticText stError = new StaticText();
			stError.setText("<p align=\"center\"><font color=\"#FF0000\">" +
				sErrorMessage + "</font></p>");
			addVisualObject(0, stError);
		}
		addVisualObject(form);		
	}
	
	private Dialog onSubmit(WebBean wb) {
		m_nAction = 3;
		return this;
	}
	
	private Dialog onCancel(WebBean wb) {
		return getParent();
	}
	
	private Dialog onResetChanges(WebBean wb) {
		m_nAction = 0;
		return this;
	}
}