package arcade.assignments;

import java.util.*;
import arcade.util.*;
import arcade.util.html.*;
import arcade.ccim.courseinstancemanagement.*;
import arcade.usermanagement.*;

import arcade.util.i18n.MessagesBundle;
import arcade.ArcadeConstants;

public class ManageAssignmentsProposalsDlg extends Dialog {
	
	transient private int m_nIDCourseInstance = -1;
	transient private int m_nIDAssignmentProposal = -1;
	
	public ManageAssignmentsProposalsDlg(String sAction) {
		super("ManageAssignmentsProposalsDlg", sAction, "");
	}
	
	protected void onMapVar() {
		mapVar("m_nIDCourseInstance", "CourseInstance", c_VAR_IN);
		mapVar("m_nIDAssignmentProposal", "AssignmentProposal", c_VAR_IN);
	}
	
	protected void paint(WebBean wb) {
		try {
			if (!isValid("m_nIDCourseInstance")) m_nIDCourseInstance = -1;
			
			addVisualObject(new Header(MessagesBundle.getString("key_assignments_proposals", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),"center"));
			
			InputForm form = new InputForm(this);
			
			addHiddenField("form", "1");
			String sLetter = wb.getRequestParameter("letter");
			char cLetter = LetterSelector.C_NO_LETTER_SELECTED;
			if (sLetter.length() > 0) cLetter = sLetter.toCharArray()[0];
			addHiddenField("letter", "");
			
			Select select = new Select("CourseInstance", 1);
			select.addOption("", "-1", (m_nIDCourseInstance == -1) ? true : false);
			
			User user = wb.getLoggedUser(wb.m_request.getSession());
			CourseInstance ci;
			if (user.isInGroup(Group.C_INSTRUCTOR_NAME)) {
				CourseInstanceArrayList list;
				list = ManageAssignmentsUI.getCourseInstances(true, ManageAssignmentsUI.C_PRIV_WRITE, wb.getLoggedUserID());
				for (int nIndex = 0, nCount = list.getCourseInstancesCount(); nIndex < nCount; nIndex++) {
					ci = list.getCourseInstanceAt(nIndex);
					select.addOption(ci.getName(), String.valueOf(ci.getID()), (m_nIDCourseInstance == ci.getID()) ? true : false);
				}
			}
			else {
				ArrayList list;
				list = ManageAssignmentsUI.getCourseInstancesForStudent((int)wb.getLoggedUserID(),true);
				for (int nIndex = 0, nCount = list.size(); nIndex < nCount; nIndex++) {
					ci = (CourseInstance)list.get(nIndex);
					select.addOption(ci.getName(), String.valueOf(ci.getID()), (m_nIDCourseInstance == ci.getID()) ? true : false);
				}
			}			
			addHandler(select, "OnCourseInstanceSelect", c_EVENT_ON_CHANGE);
			form.addInput(MessagesBundle.getString("key_course_instance", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
			 	select);
			 	
			addVisualObject(form);
			addVisualObject(new StaticText ("<BR>"));
			LetterSelector letterSelector = new LetterSelector ("javascript: refresh(); //");
			letterSelector.addOnClick("document.forms[0].letter.value=cLetter;");
			letterSelector.setSelectedLetter(cLetter);
			addVisualObject(letterSelector);
			Table table = new Table("AssignmentProposal");
			table.addBorderTable(false);
			table.setWidth("100%");
			table.setColumnLabels(new String[]{
							MessagesBundle.getString("key_assignement_name", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
							MessagesBundle.getString("key_type", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
							MessagesBundle.getString("key_description", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())
							});
			table.setColumnWidths(new int[]{250, 50, 0});
			ArrayList ass = null;
			String sWhere = "";
			if (cLetter != LetterSelector.C_NO_LETTER_SELECTED)
				sWhere = " AND (NAME like('" + Character.toUpperCase(cLetter) + "%') or " +
					"NAME like('" + Character.toLowerCase(cLetter) + "%'))";
			if (m_nIDCourseInstance > 0) ass = Util.getAssignmentsProposals(user, wb.getLoggedUserID(), m_nIDCourseInstance, true, sWhere);
			for (int nAssignment = 0; nAssignment < ((m_nIDCourseInstance > 0) ? ass.size() : 0); nAssignment++) {
				try {
					AssignmentProposal ap = (AssignmentProposal)ass.get(nAssignment);
					table.addRow(ap.getID(), new String[]
						{ap.getName(), (ap.getType() == null) ? "" : ap.getType(), (ap.getDescription() == null)? "" : ap.getDescription()});
				} catch (Exception ex) {
					Exception e = new Exception ("Assignments::render: exception at reading assignments " + nAssignment + "<br>" + ex);
					throw e;
				}
			}
			boolean instructor = false;
			boolean student = false;
			User loggedUser = wb.getLoggedUser(wb.getSession());
			if (loggedUser.isInGroup(Group.C_INSTRUCTOR_NAME)) instructor = true;
			if (loggedUser.isInGroup(Group.C_STUDENT_NAME)) student = true;
			if (instructor) {
				table.setOnSelect("document.forms[0].AssignmentProposal.value=this.value; bnEditAss.disabled = false;");
			} else {
				table.setOnSelect("document.forms[0].AssignmentProposal.value=this.value; bnDeleteAss.disabled = false; bnEditAss.disabled = false;");
			}
			addVisualObject(table);
			
			addVisualObject(new StaticText ("<br>"));
			ButtonBar bar = new ButtonBar();
			bar.setAlignment("center");
			Button bnCreateAss = new Button(
								Button.C_TYPE_BUTTON,
								MessagesBundle.getString("key_create", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
								null),
				bnDeleteAss = new Button(Button.C_TYPE_BUTTON,
								MessagesBundle.getString("key_delete", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
								null),
				bnEditAss = new Button(Button.C_TYPE_BUTTON,
								 MessagesBundle.getString("key_edit", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
								 null);
			
			addHandler(bnCreateAss, "OnCreateAss", c_EVENT_ON_CLICK);
			addHandler(bnDeleteAss, "OnDeleteAss", c_EVENT_ON_CLICK);
			addHandler(bnEditAss, "OnEditAss", c_EVENT_ON_CLICK);
			if (m_nIDCourseInstance < 1) bnCreateAss.Disable();
			bnEditAss.Disable();
			bnDeleteAss.Disable(); 
			bnDeleteAss.setName("bnDeleteAss");
			bnEditAss.setName("bnEditAss");
			
			if (student) bar.addButton(bnCreateAss); 
			bar.addButton(bnEditAss);
			if (student) bar.addButton(bnDeleteAss); 
			
			addVisualObject(bar);
		
		} catch (Exception ex) {}
	}
	
	private Dialog OnCourseInstanceSelect(WebBean wb) {
		m_nIDAssignmentProposal = -1;
		return this;
	}
	private Dialog OnCreateAss(WebBean wb) {
		AssignmentProposalDlg apd = new AssignmentProposalDlg(this, AssignmentProposalDlg.c_TYPE_CREATE);
		return apd;
	}
	private Dialog OnEditAss(WebBean wb) {
		AssignmentProposalDlg apd = new AssignmentProposalDlg(this, AssignmentProposalDlg.c_TYPE_EDIT);
		return apd;
	}
	private Dialog OnDeleteAss(WebBean wb) {
		AssignmentProposalDlg apd = new AssignmentProposalDlg(this, AssignmentProposalDlg.c_TYPE_DELETE);
		return apd;
	}
}
