package arcade.assignments;

import java.util.ArrayList;

import arcade.ArcadeConstants;
import arcade.ccim.courseinstancemanagement.CourseInstance;
import arcade.ccim.courseinstancemanagement.CourseInstanceArrayList;
import arcade.util.Dialog;
import arcade.util.WebBean;
import arcade.util.html.Button;
import arcade.util.html.ButtonBar;
import arcade.util.html.Header;
import arcade.util.html.InputForm;
import arcade.util.html.Select;
import arcade.util.html.StaticText;
import arcade.util.html.Table;
import arcade.util.i18n.MessagesBundle;

public class ManageCalendarEventsDlg extends Dialog {
	transient private int m_nIDCourseInstace = -1;
	transient private int m_nIDCalendarEvent = -1;
	
	public ManageCalendarEventsDlg(Dialog dlg) {
		super("ManageCalendarEventsDlg", dlg);
	}
	
	public ManageCalendarEventsDlg(String sAction) {
		super("ManageCalendarEventsDlg", sAction, "");
	}
	
	protected void onMapVar() {
		mapVar("m_nIDCourseInstace", "CourseIndex", c_VAR_IN);
		mapVar("m_nIDCalendarEvent", "CalendarEvent", c_VAR_IN);
	}
	
	protected void paint(WebBean wb) {
		try {
			if (!isValid("m_nIDCourseInstace")) m_nIDCourseInstace = -1;
			addVisualObject(new Header(
				MessagesBundle.getString("key_manage_calendar_events", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
			 	"center"));
			InputForm form = new InputForm(this);
			addHiddenField("form", "1");
			Select select = new Select("CourseIndex", 1);
			select.addOption("", "-1", (m_nIDCourseInstace == -1) ? true : false);
			CourseInstanceArrayList list = ManageAssignmentsUI.getCourseInstances(true, ManageAssignmentsUI.C_PRIV_WRITE, wb.getLoggedUserID());
			CourseInstance ci;
			for (int nIndex = 0, nCount = list.getCourseInstancesCount(); nIndex < nCount; nIndex++) {
				ci = list.getCourseInstanceAt(nIndex);
				select.addOption(ci.getName(), String.valueOf(ci.getID()),
						(m_nIDCourseInstace == ci.getID()) ? true : false);
			}
			addHandler(select, "OnCourseInstanceSelect", c_EVENT_ON_CHANGE);
			form.addInput(
				MessagesBundle.getString("key_course_instance", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
			 	select);
			addVisualObject(form);
			 	
			Table table = new Table("CalendarEvent");
			table.addBorderTable(false);
			table.setWidth("100%");
			table.setColumnLabels(new String[]{
							  MessagesBundle.getString("key_type", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),							  
							  MessagesBundle.getString("key_name", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),							  
							  MessagesBundle.getString("key_start_date", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
							  MessagesBundle.getString("key_end_date", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
							  MessagesBundle.getString("key_description", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale())
							  });
			table.setColumnWidths(new int[]{100, 100, 100, 100, 0});
			
			ArrayList calEv = null;
			if (m_nIDCourseInstace > 0) calEv = ManageAssignmentsUI.getCalendarEvents(m_nIDCourseInstace, true);
			
			for (int nCalendarEvent = 0; nCalendarEvent < ((m_nIDCourseInstace > 0) ? calEv.size() : 0); nCalendarEvent++)
			try {
				CalendarEvent ce = (CalendarEvent)calEv.get(nCalendarEvent);
				table.addRow(ce.getID(), new String[]
					{ce.getType(),
					(ce.getName() == null)? "" : ce.getName(), 
					ce.getStartDate().toString(),
					ce.getEndDate().toString(),					 
					(ce.getDescription() == null)? "" : ce.getDescription()});
			} catch (Exception ex) {
				Exception e = new Exception ("CalendarEvents::exception at reading calendar events " + nCalendarEvent + "<br>" + ex);
				throw e;
			}
			
			table.setOnSelect("document.forms[0].CalendarEvent.value=this.value; bnDeleteCalendarEvent.disabled = false; bnEditCalendarEvent.disabled = false;");
			addVisualObject(table);
			
			addVisualObject(new StaticText ("<br>"));
			ButtonBar bar = new ButtonBar();
			bar.setAlignment("center");
			Button bnCreateCalendarEvent = new Button(
								Button.C_TYPE_BUTTON,
								MessagesBundle.getString("key_create", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
								null),
				bnDeleteCalendarEvent = new Button(Button.C_TYPE_BUTTON,
								MessagesBundle.getString("key_delete", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
								null),
				bnEditCalendarEvent = new Button(Button.C_TYPE_BUTTON,
								 MessagesBundle.getString("key_edit", ArcadeConstants.DEFAULT_BUNDLE, wb.getUserLocale()),
								 null);
			addHandler(bnCreateCalendarEvent, "OnCreateCalendarEvent", c_EVENT_ON_CLICK);
			addHandler(bnDeleteCalendarEvent, "OnDeleteCalendarEvent", c_EVENT_ON_CLICK);
			addHandler(bnEditCalendarEvent, "OnEditCalendarEvent", c_EVENT_ON_CLICK);
			if (m_nIDCourseInstace < 1) bnCreateCalendarEvent.Disable();
			bnDeleteCalendarEvent.Disable(); bnEditCalendarEvent.Disable();
			bnDeleteCalendarEvent.setName("bnDeleteCalendarEvent");
			bnEditCalendarEvent.setName("bnEditCalendarEvent");
			bar.addButton(bnCreateCalendarEvent);
			bar.addButton(bnEditCalendarEvent);
			bar.addButton(bnDeleteCalendarEvent);
			addVisualObject(bar);	
		} catch (Exception ex) {
			ex.printStackTrace();
		}		
	}
	
	private Dialog OnCourseInstanceSelect(WebBean wb) {
		m_nIDCalendarEvent = -1;
		return this;
	}
	
	private Dialog OnCreateCalendarEvent(WebBean wb) {
		CalendarEventDlg ce = new CalendarEventDlg(this, CalendarEventDlg.c_TYPE_CREATE);
		return ce;
	}
	
	private Dialog OnDeleteCalendarEvent(WebBean wb) {
		CalendarEventDlg ce = new CalendarEventDlg(this, CalendarEventDlg.c_TYPE_DELETE);
		return ce;
	}
	
	private Dialog OnEditCalendarEvent(WebBean wb) {
		CalendarEventDlg ce = new CalendarEventDlg(this, CalendarEventDlg.c_TYPE_EDIT);
		return ce;
	}
}